@@ -7,8 +7,6 @@ from typing import TYPE_CHECKING, Optional, Dict, Set, ClassVar, Any, Tuple, Uni
 if TYPE_CHECKING:
     from worlds._bizhawk.context import BizHawkClientContext, BizHawkClientCommandProcessor
 
-# TODO : For Stun/Freeze Trap : SpikeState2 set to 0x58 freeze the movement of Spike, if I find a way to stop position update could be good
-# TODO : For Spin Trap : SpikeState2 set to 0x66
 class ApeEscapeMemoryInput:
     def __init__(self, bizhawk_client_context: "BizHawkClientContext"):
         self.bizhawk_client_context = bizhawk_client_context
@@ -76,7 +74,7 @@ class ApeEscapeMemoryInput:
         # --- MonkeyMashHandler class ---
 
 class MonkeyMashHandler:
-    MAX_TRAP_DURATION = 30  # Maximum duration for the trap in seconds
+    MAX_TRAP_DURATION = 20  # Maximum duration for the trap in seconds
 
     def __init__(self, bizhawk_client_context: Union["BizHawkClientContext", None]):
         self.bizhawk_client_context = bizhawk_client_context
@@ -92,7 +90,7 @@ class MonkeyMashHandler:
         self.input_controller = ApeEscapeMemoryInput(
             self.bizhawk_client_context) if self.bizhawk_client_context else None
 
-        self.input_frequency = 0.7      # Time between NEW random inputs (e.g., generate new input every 0.7s)
+        self.input_frequency = 1      # Time between NEW random inputs (e.g., generate new input every 1s)
         self.last_input_time = 0
 
         self.input_hold_time = 0.5      # How long the inputs will be pressed
@@ -320,13 +318,14 @@ class StunTrapHandler:
         self.bizhawk_context = bizhawk_client_context.bizhawk_ctx if bizhawk_client_context else None
 
         self.is_active = False          # True if Rainbow Cookie effects are currently active
+        self.RoomType = "Special"
         self.duration = 0               # The initial or current duration set for the cookie
         self.remaining_time = 0         # How much time is left for the effects
         self.last_update = 0            # Timestamp of the last update, for calculating elapsed time
         self.pause = False              # Flag to pause the cookie's timer/effects
         self.lastspikestate = 0x00      # To store last SpikeState on activation
         self.sentMessage = True         # To track if the last activation sent a Bizhawk message on expiration
-    async def activate_StunTrap(self, duration_seconds: int,lastspikestate):
+    async def activate_StunTrap(self, duration_seconds: int,lastspikestate,currentRoom):
         """
         Activates the Stun Trap effects.
         If already active, extends the duration up to MAX_DURATION.
@@ -340,11 +339,27 @@ class StunTrapHandler:
             self.lastspikestate = lastspikestate
         # First activation
         self.is_active = True
+
+        read_list = []
+        read_list += [(RAM.SpecialRoom_CameraMode, 1, "MainRAM"),(RAM.Boss_CameraMode, 1, "MainRAM"),(RAM.Inside_CameraMode, 1, "MainRAM")]
+        CameraMode_reads = await bizhawk.read(self.bizhawk_context, read_list)
+        isSpecialRoom = int.from_bytes(CameraMode_reads[0], byteorder="little") == 0x01
+        isBossRoom = int.from_bytes(CameraMode_reads[1], byteorder="little") == 0x01
+        isInside  = int.from_bytes(CameraMode_reads[2], byteorder="little") == 0x01
+        if isSpecialRoom:
+            self.RoomType = "Special"
+        elif isBossRoom:
+            self.RoomType = "Boss"
+        elif isInside:
+            self.RoomType = "Inside"
+        else:
+            self.RoomType = "Outside"
+
         self.duration = duration_seconds
         self.remaining_time = duration_seconds
         self.last_update = time.time()
         print(f"Stun Trap activated for {duration_seconds} seconds.")
-        await self._apply_effects(True) # Apply effects immediately
+        await self._apply_effects(True,currentRoom) # Apply effects immediately
         #else:
         # Activate it each time, do not extend it
         # Extend existing duration
@@ -353,7 +368,7 @@ class StunTrapHandler:
         #self.duration = self.remaining_time # Update current duration if extended
         #print(f"Stun Trap extended by {duration_seconds} seconds. Total remaining: {self.remaining_time:.2f}s (capped at {self.MAX_DURATION}s)")
         #self.sentMessage = False
-    async def _apply_effects(self, enable: bool):
+    async def _apply_effects(self, enable: bool,currentRoom):
         """
         Internal method to apply or remove the Rainbow Cookie's effects
         by writing to BizHawk memory addresses.
@@ -370,46 +385,63 @@ class StunTrapHandler:
         Spike_PosUpdates_keys = list(Spike_PosUpdatesAddress.keys())
         Spike_PosUpdates_values = list(Spike_PosUpdatesAddress.values())
 
-        for x in range(len(Spike_PosUpdates_keys)):
-            PosUpdates_values = list(Spike_PosUpdates_values[x])
-            PosUpdates_bytes = PosUpdates_values[0]
-            PosUpdates_onvalue = PosUpdates_values[1].to_bytes(PosUpdates_bytes, "little")
-            PosUpdates_offvalue = PosUpdates_values[2].to_bytes(PosUpdates_bytes, "little")
-            PosUpdates_address = (Spike_PosUpdates_keys[x])
-            if enable:
-                writes_list += [(PosUpdates_address, PosUpdates_offvalue, "MainRAM")]
-            else:
-                writes_list += [(PosUpdates_address, PosUpdates_onvalue, "MainRAM")]
+        #for x in range(len(Spike_PosUpdates_keys)):
+        #    PosUpdates_values = list(Spike_PosUpdates_values[x])
+        #    PosUpdates_bytes = PosUpdates_values[0]
+        #    PosUpdates_onvalue = PosUpdates_values[1].to_bytes(PosUpdates_bytes, "little")
+        #    PosUpdates_offvalue = PosUpdates_values[2].to_bytes(PosUpdates_bytes, "little")
+        #    PosUpdates_address = (Spike_PosUpdates_keys[x])
+        if enable:
+            writes_list += [(RAM.Spike_CanMove, 0x01.to_bytes(1,"little"), "MainRAM")]
+        else:
+            writes_list += [(RAM.Spike_CanMove, 0x00.to_bytes(1,"little"), "MainRAM")]
 
         Spike_VelocityUpdatesAddress = RAM.Spike_VelocityUpdates
         Spike_VelocityUpdates_keys = list(Spike_VelocityUpdatesAddress.keys())
         Spike_VelocityUpdates_values = list(Spike_VelocityUpdatesAddress.values())
 
-        for x in range(len(Spike_VelocityUpdates_keys)):
-            VelocityUpdates_values = list(Spike_VelocityUpdates_values[x])
-            VelocityUpdates_bytes = VelocityUpdates_values[0]
-            VelocityUpdates_onvalue = VelocityUpdates_values[1].to_bytes(VelocityUpdates_bytes, "little")
-            VelocityUpdates_offvalue = VelocityUpdates_values[2].to_bytes(VelocityUpdates_bytes, "little")
-            VelocityUpdates_address = (Spike_VelocityUpdates_keys[x])
-            if enable:
-                writes_list += [(VelocityUpdates_address, VelocityUpdates_offvalue, "MainRAM")]
-            else:
-                writes_list += [(VelocityUpdates_address, VelocityUpdates_onvalue, "MainRAM")]
+        #for x in range(len(Spike_VelocityUpdates_keys)):
+        #    VelocityUpdates_values = list(Spike_VelocityUpdates_values[x])
+        #    VelocityUpdates_bytes = VelocityUpdates_values[0]
+        #    VelocityUpdates_onvalue = VelocityUpdates_values[1].to_bytes(VelocityUpdates_bytes, "little")
+        #    VelocityUpdates_offvalue = VelocityUpdates_values[2].to_bytes(VelocityUpdates_bytes, "little")
+        #    VelocityUpdates_address = (Spike_VelocityUpdates_keys[x])
+        #    if enable:
+        #        writes_list += [(VelocityUpdates_address, VelocityUpdates_offvalue, "MainRAM")]
+        #    else:
+        #        writes_list += [(VelocityUpdates_address, VelocityUpdates_onvalue, "MainRAM")]
 
-        LastState = self.lastspikestate
+        #LastState = self.lastspikestate
 
-        InvalidLastStates = [0x80, 0x81, 0x82, 0x83, 0x84,0x2F,0x30,0x58]
+        #InvalidLastStates = [0x80, 0x81, 0x82, 0x83, 0x84,0x2F,0x30,0x58]
         # If enabling the Trap set it to 0x58, else set it to the last saved state
-        if enable:
-            Spikestate2_value = 0x58
-        else:
-            #If LastState is invalid,
-            Spikestate2_value = 0x00 if LastState in InvalidLastStates else LastState
+        #if enable:
+        #    Spikestate2_value = 0x58
+        #    CameraMode = 0x00
+        #else:
+        #    #If LastState is invalid,
+        #    Spikestate2_value = 0x00 if LastState in InvalidLastStates else LastState
+        #    CameraMode = 0x01
+        #    self.lastspikestate = 0x00
 
-            self.lastspikestate = 0x00
-        Spikestate2_bytes = list(Spikestate2_value.to_bytes(1, "little"))
+        #SpecialRooms = [30,83, 84, 87, 88, 90, 91]
+        #BossRooms = [item for item in RAM.bossListLocal.keys() if item not in SpecialRooms]
 
-        writes_list.append((RAM.spikeState2Address, Spikestate2_bytes, "MainRAM"))
+        #if self.RoomType == "Special":
+        #    CameraModeAddress = RAM.SpecialRoom_CameraMode
+        #elif self.RoomType == "Boss":
+        #    CameraModeAddress = RAM.Boss_CameraMode
+        #else:
+        #    if self.RoomType == "Inside":
+        #        CameraModeAddress = RAM.Inside_CameraMode
+        #    else:
+        #        CameraModeAddress = RAM.Outside_CameraMode
+
+        #Spikestate2_bytes = list(Spikestate2_value.to_bytes(1, "little"))
+        #CameraMode_bytes = list(CameraMode.to_bytes(1, "little"))
+
+        #writes_list.append((RAM.spikeState2Address, Spikestate2_bytes, "MainRAM"))
+        #writes_list.append((CameraModeAddress, CameraMode_bytes, "MainRAM"))
 
         try:
             await bizhawk.write(self.bizhawk_context, writes_list)
@@ -418,7 +450,7 @@ class StunTrapHandler:
             print(f"ERROR: Failed to {'apply' if enable else 'remove'} Stun Trap effects: {e}")
             raise
 
-    async def update_state_and_deactivate(self):
+    async def update_state_and_deactivate(self,currentRoom):
         """
         Updates the remaining time for the Rainbow Cookie.
         If the duration runs out, deactivates the effects.
@@ -443,4 +475,192 @@ class StunTrapHandler:
             self.remaining_time = 0
             self.is_active = False
             print("Stun Trap duration finished. Deactivating effects.")
-            await self._apply_effects(False) # Remove effects
\ No newline at end of file
+            await self._apply_effects(False,currentRoom) # Remove effects
+
+class CameraRotateHandler:
+    """
+    Manages the state and effects of the Rainbow Cookie power-up.
+    When active, makes Spike invincible and activates his golden form.
+    """
+    MAX_DURATION = 40  # Maximum duration for the Rainbow Cookie in seconds
+
+    def __init__(self, bizhawk_client_context: Union["BizHawkClientContext", None]):
+        self.bizhawk_client_context = bizhawk_client_context
+        self.bizhawk_context = bizhawk_client_context.bizhawk_ctx if bizhawk_client_context else None
+        self.chosen_side = "Left"       # Store which side the Camera Rotate is currently on
+        self.RoomType = "Special"
+        self.is_active = False          # True if effects are currently active
+        self.duration = 0               # The initial or current duration set
+        self.remaining_time = 0         # How much time is left for the effects
+        self.last_update = 0            # Timestamp of the last update, for calculating elapsed time
+        self.pause = False              # Flag to pause the timer/effects
+        self.sentMessage = True         # To track if the last activation sent a Bizhawk message on expiration
+    async def activate_camera_rotate(self, duration_seconds: int, currentRoom):
+        """
+        Activates the Rainbow Cookie effects (invincibility and golden form).
+        If already active, extends the duration up to MAX_DURATION.
+
+        Args:
+            duration_seconds (int): The number of seconds to activate/extend the cookie's effects.
+        """
+        if not self.is_active:
+            # First activation
+            self.is_active = True
+            self.duration = duration_seconds
+            self.remaining_time = duration_seconds
+            self.last_update = time.time()
+            possibleSides = ["Left","Right"]
+            self.chosen_side = possibleSides[random.randint(0,1)]
+            print(f"Camera Rotate activated for {duration_seconds} seconds.")
+
+        else:
+            ## Extend existing duration
+            new_remaining_time = self.remaining_time + duration_seconds
+            self.remaining_time = min(new_remaining_time, self.MAX_DURATION)
+            self.duration = self.remaining_time # Update current duration if extended
+            print(f"Camera Rotate extended by {duration_seconds} seconds. Total remaining: {self.remaining_time:.2f}s (capped at {self.MAX_DURATION}s)")
+
+        await self._apply_effects(True, currentRoom)  # Apply effects immediately
+        self.sentMessage = False
+    async def _apply_effects(self, enable: bool,currentRoom):
+        """
+        Internal method to apply or remove the Rainbow Cookie's effects
+        by writing to BizHawk memory addresses.
+
+        Args:
+            enable (bool): If True, enables effects; if False, disables them.
+        """
+        if self.bizhawk_context is None or self.bizhawk_context.connection_status != bizhawk.ConnectionStatus.CONNECTED:
+            print("Warning: BizHawk not connected. Cannot apply/remove Rainbow Cookie effects.")
+            return
+        read_list = []
+        read_list += [(RAM.SpecialRoom_CameraMode, 1, "MainRAM"),(RAM.Boss_CameraMode, 1, "MainRAM"),(RAM.Inside_CameraMode, 1, "MainRAM")]
+        CameraMode_reads = await bizhawk.read(self.bizhawk_context, read_list)
+        isSpecialRoom = int.from_bytes(CameraMode_reads[0], byteorder="little") == 0x01
+        isBossRoom = int.from_bytes(CameraMode_reads[1], byteorder="little") == 0x01
+        isInside  = int.from_bytes(CameraMode_reads[2], byteorder="little") == 0x01
+        if isSpecialRoom:
+            self.RoomType = "Special"
+        elif isBossRoom:
+            self.RoomType = "Boss"
+        elif isInside:
+            self.RoomType = "Inside"
+        else:
+            self.RoomType = "Outside"
+        writes_list = []
+        #SpecialRooms = [83, 84, 87, 88, 90, 91]
+        #BossRooms = [item for item in RAM.bossListLocal.keys() if item not in SpecialRooms]
+
+        #InABossRoom = False
+
+        CameraRotate_value = 0xFF if enable else 0x00
+        CameraRotate_bytes = list(CameraRotate_value.to_bytes(1, "little"))
+
+        LeftRotateAddress2 = ""
+        LeftRotateAddress = ""
+        RightRotateAddress = ""
+        RightRotateAddress2 = ""
+
+        if self.RoomType == "Special":
+            LeftRotateAddress = RAM.SpecialRoom_CameraRotateLeft
+            RightRotateAddress = RAM.SpecialRoom_CameraRotateRight
+        elif self.RoomType == "Boss":
+            LeftRotateAddress = RAM.Boss_CameraRotateLeft
+            RightRotateAddress = RAM.Boss_CameraRotateRight
+        else:
+            if self.RoomType == "Inside":
+                LeftRotateAddress = RAM.Inside_CameraRotateLeft
+                RightRotateAddress = RAM.Inside_CameraRotateLeft
+            else:
+                LeftRotateAddress = RAM.Outside_CameraRotateLeft
+                RightRotateAddress = RAM.Outside_CameraRotateRight
+
+        if self.chosen_side == "Left":
+            writes_list.append((LeftRotateAddress, CameraRotate_bytes, "MainRAM"))
+        else:
+            writes_list.append((RightRotateAddress, CameraRotate_bytes, "MainRAM"))
+
+        try:
+            await bizhawk.write(self.bizhawk_context, writes_list)
+            print(f"Camera Rotate effects {'applied' if enable else 'removed'}.")
+        except Exception as e:
+            print(f"ERROR: Failed to {'apply' if enable else 'remove'} Camera Rotate effects: {e}")
+            raise
+
+    async def update_state_and_deactivate(self,currentRoom):
+        """
+        Updates the remaining time for the Camera Rotate.
+        If the duration runs out, deactivates the effects.
+        This method should be called periodically in the main loop of the client.
+        It also re-applies the camera Rotate effect if it's lost and the cookie is active.
+        """
+        if not self.is_active:
+            return
+
+        if self.pause:
+            # If paused, don't decrement remaining_time, but update last_update
+            # to prevent a large time jump when unpaused.
+            self.last_update = time.time()
+            return
+
+        current_time = time.time()
+        elapsed_time_since_last_update = current_time - self.last_update
+        self.remaining_time -= elapsed_time_since_last_update
+        self.last_update = current_time
+
+        # Check and re-apply Rotate effect if it's not active but the effect is
+        if self.bizhawk_context and self.bizhawk_context.connection_status == bizhawk.ConnectionStatus.CONNECTED:
+            try:
+                #SpecialRooms = [83, 84, 87, 88, 90, 91]
+                #BossRooms = [item for item in RAM.bossListLocal.keys() if item not in SpecialRooms]
+                #InABossRoom = False
+                read_list = []
+                read_list += [(RAM.SpecialRoom_CameraMode, 1, "MainRAM"), (RAM.Boss_CameraMode, 1, "MainRAM"),(RAM.Inside_CameraMode, 1, "MainRAM")]
+                CameraMode_reads = await bizhawk.read(self.bizhawk_context, read_list)
+                isSpecialRoom = int.from_bytes(CameraMode_reads[0], byteorder="little") == 0x01
+                isBossRoom = int.from_bytes(CameraMode_reads[1], byteorder="little") == 0x01
+                isInside = int.from_bytes(CameraMode_reads[2], byteorder="little") == 0x01
+                if isSpecialRoom:
+                    self.RoomType = "Special"
+                elif isBossRoom:
+                    self.RoomType = "Boss"
+                elif isInside:
+                    self.RoomType = "Inside"
+                else:
+                    self.RoomType = "Outside"
+                CameraRotate_value = 0xFF
+                if self.RoomType == "Special":
+                    LeftRotateAddress = RAM.SpecialRoom_CameraRotateLeft
+                    RightRotateAddress = RAM.SpecialRoom_CameraRotateRight
+                elif self.RoomType == "Boss":
+                    LeftRotateAddress = RAM.Boss_CameraRotateLeft
+                    RightRotateAddress = RAM.Boss_CameraRotateRight
+                else:
+                    if self.RoomType == "Inside":
+                        LeftRotateAddress = RAM.Inside_CameraRotateLeft
+                        RightRotateAddress = RAM.Inside_CameraRotateLeft
+                    else:
+                        LeftRotateAddress = RAM.Outside_CameraRotateLeft
+                        RightRotateAddress = RAM.Outside_CameraRotateRight
+
+                if self.chosen_side == "Left":
+                    CameraRotateAddress = LeftRotateAddress
+                else:
+                    CameraRotateAddress = RightRotateAddress
+
+                # Read the current value of the chosen CameraRotate address
+                current_camera_rotate_bytes = await bizhawk.read(self.bizhawk_context, [(CameraRotateAddress, 1, "MainRAM")])
+                current_camera_rotate = int.from_bytes(current_camera_rotate_bytes[0], byteorder="little")
+
+                if current_camera_rotate is not None and current_camera_rotate != 0xFF:
+                    print("Camera Rotate active, but rotate effect lost. Reapplying...")
+                    await self._apply_effects(True,currentRoom)
+            except Exception as e:
+                print(f"ERROR: Failed to read rotate form address for reapplication: {e}")
+                # Log error but don't stop the loop for this non-critical re-application check
+
+        if self.remaining_time <= 0:
+            self.remaining_time = 0
+            self.is_active = False
+            print("Camera Rotate duration finished. Deactivating effects.")
+            await self._apply_effects(False,currentRoom) # Remove effects
