@@ -1,19 +1,22 @@
-from worlds.apeescape.Strings import AEDoor
+from .Strings import AEDoor
 
 
 class RAM:
 # ===================== Locations/Checks =====================
     monkeyListGlobal = {
+        # FF
         1: 0x0DF828,
         2: 0x0DF829,
         3: 0x0DF82A,
         4: 0x0DF82B,
+        # PO
         5: 0x0DF830,
         6: 0x0DF831,
         8: 0x0DF832,
         7: 0x0DF833,
         10: 0x0DF834,
         9: 0x0DF835,
+        # ML
         11: 0x0DF838,
         12: 0x0DF839,
         13: 0x0DF83A,
@@ -21,6 +24,7 @@ class RAM:
         15: 0x0DF840,
         14: 0x0DF841,
         16: 0x0DF848,
+        # TJ
         18: 0x0DF850,
         19: 0x0DF851,
         20: 0x0DF852,
@@ -35,6 +39,7 @@ class RAM:
         26: 0x0DF86A,
         28: 0x0DF870,
         27: 0x0DF871,
+        # DR
         33: 0x0DF878,
         37: 0x0DF879,
         42: 0x0DF87A,
@@ -48,6 +53,7 @@ class RAM:
         39: 0x0DF890,
         40: 0x0DF891,
         44: 0x0DF892,
+        # CR
         51: 0x0DF898,
         49: 0x0DF899,
         45: 0x0DF8A0,
@@ -56,6 +62,7 @@ class RAM:
         50: 0x0DF8AA,
         48: 0x0DF8B0,
         52: 0x0DF8B1,
+        # CB
         53: 0x0DF8C0,
         54: 0x0DF8C1,
         55: 0x0DF8C2,
@@ -64,6 +71,7 @@ class RAM:
         60: 0x0DF8C9,
         58: 0x0DF8CA,
         59: 0x0DF8CB,
+        # CC
         61: 0x0DF8D0,
         62: 0x0DF8D1,
         63: 0x0DF8D2,
@@ -72,6 +80,7 @@ class RAM:
         67: 0x0DF8D9,
         66: 0x0DF8DA,
         68: 0x0DF8DB,
+        # DI
         70: 0x0DF8E0,
         69: 0x0DF8E1,
         77: 0x0DF8E8,
@@ -83,12 +92,14 @@ class RAM:
         75: 0x0DF8F3,
         76: 0x0DF8F4,
         79: 0x0DF8F8,
+        # SM
         80: 0x0DF908,
         81: 0x0DF909,
         84: 0x0DF90A,
         82: 0x0DF90B,
         83: 0x0DF90C,
         85: 0x0DF90D,
+        # FR
         86: 0x0DF910,
         87: 0x0DF911,
         91: 0x0DF918,
@@ -98,6 +109,7 @@ class RAM:
         88: 0x0DF920,
         89: 0x0DF921,
         90: 0x0DF922,
+        # HS
         95: 0x0DF928,
         96: 0x0DF929,
         99: 0x0DF92A,
@@ -107,6 +119,7 @@ class RAM:
         103: 0x0DF932,
         97: 0x0DF938,
         98: 0x0DF939,
+        # ST
         104: 0x0DF948,
         105: 0x0DF949,
         106: 0x0DF94A,
@@ -119,6 +132,7 @@ class RAM:
         113: 0x0DF958,
         111: 0x0DF959,
         112: 0x0DF95A,
+        # WSW
         116: 0x0DF960,
         117: 0x0DF961,
         118: 0x0DF968,
@@ -129,6 +143,7 @@ class RAM:
         122: 0x0DF979,
         124: 0x0DF980,
         125: 0x0DF981,
+        # CrC
         127: 0x0DF988,
         136: 0x0DF989,
         126: 0x0DF98A,
@@ -149,29 +164,32 @@ class RAM:
         143: 0x0DF9B8,
         144: 0x0DF9B9,
         145: 0x0DF9BA,
+        # CP
         146: 0x0DF9C8,
         147: 0x0DF9C9,
         148: 0x0DF9CA,
         149: 0x0DF9CB,
-        152: 0x0DF9D2,
-        151: 0x0DF9D1,
         150: 0x0DF9D0,
+        151: 0x0DF9D1,
+        152: 0x0DF9D2,
         153: 0x0DF9D8,
         154: 0x0DF9D9,
         155: 0x0DF9DA,
         156: 0x0DF9DB,
         157: 0x0DF9DC,
         158: 0x0DF9DD,
+        # SF
         159: 0x0DF9E0,
         160: 0x0DF9E1,
         161: 0x0DF9E8,
         162: 0x0DF9F0,
-        168: 0x0DFA10,
-        164: 0x0DF9F9,
-        167: 0x0DFA09,
-        166: 0x0DFA08,
-        165: 0x0DF9FA,
         163: 0x0DF9F8,
+        164: 0x0DF9F9,
+        165: 0x0DF9FA,
+        166: 0x0DFA08,
+        167: 0x0DFA09,
+        168: 0x0DFA10,
+        # TVT
         169: 0x0DFA18,
         170: 0x0DFA20,
         171: 0x0DFA21,
@@ -184,6 +202,7 @@ class RAM:
         178: 0x0DFA39,
         179: 0x0DFA3A,
         180: 0x0DFA3B,
+        # MM
         181: 0x0DFA60,
         182: 0x0DFA78,
         183: 0x0DFA80,
@@ -194,20 +213,20 @@ class RAM:
         188: 0x0DFA8A,
         189: 0x0DFA8B,
         190: 0x0DFA90,
+        191: 0x0DFA98,
         192: 0x0DFA99,
         193: 0x0DFAA0,
         194: 0x0DFAA1,
         196: 0x0DFAA3,
         195: 0x0DFAA2,
         197: 0x0DFAA8,
+        198: 0x0DFAA9,
+        199: 0x0DFAB0,
+        200: 0x0DFAB1,
         201: 0x0DFAC0,
         202: 0x0DFAC1,
         203: 0x0DFAC2,
         204: 0x0DFAC8,
-        198: 0x0DFAA9,
-        199: 0x0DFAB0,
-        200: 0x0DFAB1,
-        191: 0x0DFA98
     }
 
     monkeyListLocal = {
@@ -592,6 +611,391 @@ class RAM:
 
     }
 
+# List of temporary table that monkeys caught status get saved while traversing level.
+# Get's written when going into a transition
+# When exiting the level,it save them into global values
+    monkeyListTempLocal = {
+        1: {  # 1-1
+            1: 0x0DFE00,
+            3: 0x0DFE01,
+            2: 0x0DFE02,
+            4: 0x0DFE03
+        },
+        2: {  # 1-2
+            5: 0x0DFE00,
+            6: 0x0DFE01,
+            7: 0x0DFE03,
+            10: 0x0DFE04,
+            9: 0x0DFE05,
+            8: 0x0DFE02
+        },
+        3: {  # 1-3
+            11: 0x0DFE00,
+            12: 0x0DFE01,
+            17: 0x0DFE03,
+            13: 0x0DFE02
+        },
+        4: {  # volcano
+            14: 0x0DFE19,
+            15: 0x0DFE18
+        },
+        5: {  # triceratops
+            16: 0x0DFE30
+        },
+        6: {  # 2-1
+            18: 0x0DFE00,
+            19: 0x0DFE01,
+            20: 0x0DFE02
+        },
+        7: {  # mushroom area
+            29: 0x0DFE18,
+            30: 0x0DFE1A,
+            31: 0x0DFE19
+        },
+        8: {  # fish room
+            23: 0x0DFE32,
+            21: 0x0DFE30,
+            22: 0x0DFE31
+        },
+        9: {  # tent/vine room
+            24: 0x0DFE49,
+            25: 0x0DFE48,
+            26: 0x0DFE4A
+        },
+        10: {  # boulder room
+            27: 0x0DFE61,
+            28: 0x0DFE60
+        },
+        11: {  # 2-2
+            32: 0x0DFE04,
+            33: 0x0DFE00,
+            34: 0x0DFE03,
+            37: 0x0DFE01,
+            42: 0x0DFE02
+        },
+        12: {  # fan basement
+            35: 0x0DFE18,
+            36: 0x0DFE19
+        },
+        13: {  # obelisk inside
+            38: 0x0DFE32,
+            41: 0x0DFE30,
+            43: 0x0DFE31
+        },
+        14: {  # water basement
+            39: 0x0DFE48,
+            40: 0x0DFE49,
+            44: 0x0DFE4A
+        },
+        15: {  # 2-3
+            49: 0x0DFE01,
+            51: 0x0DFE00
+        },
+        16: {  # side room
+            45: 0x0DFE18
+        },
+        17: {  # main ruins
+            47: 0x0DFE30,
+            50: 0x0DFE32,
+            46: 0x0DFE31
+        },
+        18: {  # pillar room
+            48: 0x0DFE48,
+            52: 0x0DFE49
+        },
+        19: {  # 3-1
+
+        },
+        20: {  # 4-1
+            53: 0x0DFE00,
+            54: 0x0DFE01,
+            55: 0x0DFE02,
+            56: 0x0DFE03
+        },
+        21: {  # second room
+            57: 0x0DFE18,
+            58: 0x0DFE1A,
+            59: 0x0DFE1B,
+            60: 0x0DFE19
+        },
+        22: {  # 4-2
+            61: 0x0DFE00,
+            62: 0x0DFE01,
+            63: 0x0DFE02,
+            64: 0x0DFE03
+        },
+        23: {  # second room
+            65: 0x0DFE18,
+            67: 0x0DFE19,
+            68: 0x0DFE1B,
+            66: 0x0DFE1A
+        },
+        24: {  # 4-3
+            69: 0x0DFE01,
+            70: 0x0DFE00
+        },
+        25: {  # stomach
+            71: 0x0DFE19,
+            77: 0x0DFE18,
+            78: 0x0DFE1A
+        },
+        26: {  # gallery/boulder
+            72: 0x0DFE30,
+            73: 0x0DFE31,
+            74: 0x0DFE32,
+            75: 0x0DFE33,
+            76: 0x0DFE34
+        },
+        27: {  # tentacle room
+            79: 0x0DFE48
+        },
+        28: {  # slide room
+
+        },
+        29: {  # 5-1
+            80: 0x0DFE00,
+            81: 0x0DFE01,
+            84: 0x0DFE02,
+            83: 0x0DFE04,
+            85: 0x0DFE05,
+            82: 0x0DFE03
+        },
+        30: {  # 5-2
+            86: 0x0DFE00,
+            87: 0x0DFE01
+        },
+        31: {  # water room
+            91: 0x0DFE18,
+            92: 0x0DFE1A,
+            93: 0x0DFE19,
+            94: 0x0DFE1B
+        },
+        32: {  # caverns
+            88: 0x0DFE30,
+            90: 0x0DFE32,
+            89: 0x0DFE31
+        },
+        33: {  # 5-3
+            95: 0x0DFE00,
+            96: 0x0DFE01,
+            99: 0x0DFE02,
+            100: 0x0DFE03
+        },
+        34: {  # hot spring
+            101: 0x0DFE18,
+            102: 0x0DFE19,
+            103: 0x0DFE1A
+        },
+        35: {  # polar bear cave
+            98: 0x0DFE31,
+            97: 0x0DFE30
+        },
+        36: {  # 6-1
+
+        },
+        37: {  # 7-1
+            104: 0x0DFE00,
+            105: 0x0DFE01,
+            106: 0x0DFE02,
+            107: 0x0DFE03
+        },
+        38: {  # temple
+            108: 0x0DFE1A,
+            110: 0x0DFE19,
+            109: 0x0DFE18,
+            114: 0x0DFE1B,
+            115: 0x0DFE1C
+        },
+        39: {  # well
+            111: 0x0DFE31,
+            112: 0x0DFE32,
+            113: 0x0DFE30
+        },
+        40: {  # 7-2
+            116: 0x0DFE00,
+            117: 0x0DFE01
+        },
+        41: {  # gong room
+            118: 0x0DFE18,
+            119: 0x0DFE19,
+            120: 0x0DFE1A
+        },
+        42: {  # middle room
+            123: 0x0DFE30
+        },
+        43: {  # obstacle course
+            122: 0x0DFE49,
+            121: 0x0DFE48,
+        },
+        44: {  # barrel room
+            124: 0x0DFE60,
+            125: 0x0DFE61
+        },
+        45: {  # 7-3
+            126: 0x0DFE02,
+            127: 0x0DFE00,
+            128: 0x0DFE03,
+            137: 0x0DFE04,
+            136: 0x0DFE01
+        },
+        46: {  # castle main
+            129: 0x0DFE18,
+            131: 0x0DFE1A,
+            130: 0x0DFE19,
+            132: 0x0DFE1B
+        },
+        47: {  # flooded basement
+            133: 0x0DFE30,
+            134: 0x0DFE31,
+            135: 0x0DFE32
+        },
+        49: {  # button room
+            139: 0x0DFE61,
+            138: 0x0DFE60
+        },
+        50: {  # elevator room
+            140: 0x0DFE78,
+            141: 0x0DFE79,
+            142: 0x0DFE7A
+        },
+        51: {  # bell tower
+            145: 0x0DFE92,
+            144: 0x0DFE91,
+            143: 0x0DFE90
+        },
+        52: {
+
+        },
+        53: {  # 8-1
+            146: 0x0DFE00,
+            149: 0x0DFE03,
+            147: 0x0DFE01,
+            148: 0x0DFE02
+        },
+        54: {  # sewers front
+            151: 0x0DFE19,
+            152: 0x0E5A1A,
+            150: 0x0DFE18
+        },
+        55: {  # barrel room
+            155: 0x0DFE32,
+            153: 0x0DFE30,
+            156: 0x0DFE33,
+            157: 0x0DFE34,
+            154: 0x0DFE31,
+            158: 0x0DFE35
+        },
+        56: {  # 8-2
+            159: 0x0DFE00,
+            160: 0x0DFE01
+        },
+        57: {  # main factory
+            161: 0x0DFE18
+        },
+        58: {  # rc car room
+            162: 0x0DFE30
+        },
+        59: {  # lava room
+            163: 0x0DFE48,
+            164: 0x0DFE49,
+            165: 0x0DFE4A
+        },
+        60: {
+
+        },
+        61: {  # conveyor room
+            166: 0x0DFE78,
+            167: 0x0DFE79
+        },
+        62: {  # mech room
+            168: 0x0DFE90
+        },
+        63: {  # 8-3
+            169: 0x0DFE00
+        },
+        64: {  # water basement
+            171: 0x0DFE19,
+            170: 0x0DFE18
+        },
+        65: {  # lobby
+            172: 0x0DFE30,
+            173: 0x0DFE31
+        },
+        66: {  # tank room
+            174: 0x0DFE48,
+            175: 0x0DFE49,
+            176: 0x0DFE4A
+        },
+        67: {  # fan room
+            177: 0x0DFE60,
+            179: 0x0DFE62,
+            180: 0x0DFE63,
+            178: 0x0DFE61
+        },
+        68: {
+
+        },
+        69: {  # MM Lobby
+
+        },
+        71: {
+
+        },
+        72: {  # coaster entry
+            181: 0x0DFE00
+        },
+        73: {  # coaster 1
+
+        },
+        74: {  # coaster 2
+
+        },
+        75: {  # haunted house
+            182: 0x0DFE48
+        },
+        76: {  # coffin room
+            183: 0x0DFE60,
+            184: 0x0DFE61,
+            185: 0x0DFE62
+        },
+        77: {  # western land
+            187: 0x0DFE01,
+            186: 0x0DFE00,
+            188: 0x0DFE02,
+            189: 0x0DFE03
+        },
+        78: {  # crater
+            190: 0x0DFE00
+        },
+        79: {  # outside castle
+            192: 0x0DFE19,
+            191: 0x0DFE18
+        },
+        80: {  # castle main
+            194: 0x0DFE31,
+            195: 0x0DFE32,
+            196: 0x0DFE33,
+            193: 0x0DFE30
+        },
+        81: {  # inside climb
+            197: 0x0DFE48,
+            198: 0x0DFE49
+        },
+        82: {  # outside climb
+            199: 0x0DFE60,
+            200: 0x0DFE61
+        },
+        84: {  # Monkey head
+            201: 0x0DFE90,
+            202: 0x0DFE91,
+            203: 0x0DFE92
+        },
+        85: {  # side entry
+            204: 0x0DFEA8
+        }
+
+    }
+
     localMonkeyHitbox = {
         #Array order -> LocalMonkeyAddress : HitboxAddress
         0x0E557A : 0x0E5525,
@@ -603,6 +1007,291 @@ class RAM:
 
     }
 
+    localmonkeyCatchHitbox = {
+        # FF
+        1: 0x0DF828,
+        2: 0x0DF829,
+        3: 0x0DF82A,
+        4: 0x0DF82B,
+        # PO
+        5: 0x0DF830,
+        6: 0x0DF831,
+        8: 0x0DF832,
+        7: 0x0DF833,
+        10: 0x0DF834,
+        9: 0x0DF835,
+        # ML
+        11: 0x0DF838,
+        12: 0x0DF839,
+        13: 0x0DF83A,
+        17: 0x0DF83B,
+        15: 0x0DF840,
+        14: 0x0DF841,
+        16: 0x0DF848,
+        # TJ
+        18: 0x0DF850,
+        19: 0x0DF851,
+        20: 0x0DF852,
+        29: 0x0DF858,
+        31: 0x0DF859,
+        30: 0x0DF85A,
+        21: 0x0DF860,
+        22: 0x0DF861,
+        23: 0x0DF862,
+        25: 0x0DF868,
+        24: 0x0DF869,
+        26: 0x0DF86A,
+        28: 0x0DF870,
+        27: 0x0DF871,
+        # DR
+        33: 0x0DF878,
+        37: 0x0DF879,
+        42: 0x0DF87A,
+        34: 0x0DF87B,
+        32: 0x0DF87C,
+        35: 0x0DF880,
+        36: 0x0DF881,
+        41: 0x0DF888,
+        43: 0x0DF889,
+        38: 0x0DF88A,
+        39: 0x0DF890,
+        40: 0x0DF891,
+        44: 0x0DF892,
+        # CR
+        51: 0x0DF898,
+        49: 0x0DF899,
+        45: 0x0DF8A0,
+        47: 0x0DF8A8,
+        46: 0x0DF8A9,
+        50: 0x0DF8AA,
+        48: 0x0DF8B0,
+        52: 0x0DF8B1,
+        # CB
+        53: 0x0DF8C0,
+        54: 0x0DF8C1,
+        55: 0x0DF8C2,
+        56: 0x0DF8C3,
+        57: 0x0DF8C8,
+        60: 0x0DF8C9,
+        58: 0x0DF8CA,
+        59: 0x0DF8CB,
+        # CC
+        61: 0x0DF8D0,
+        62: 0x0DF8D1,
+        63: 0x0DF8D2,
+        64: 0x0DF8D3,
+        65: 0x0DF8D8,
+        67: 0x0DF8D9,
+        66: 0x0DF8DA,
+        68: 0x0DF8DB,
+        # DI
+        70: 0x0DF8E0,
+        69: 0x0DF8E1,
+        77: 0x0DF8E8,
+        71: 0x0DF8E9,
+        78: 0x0DF8EA,
+        72: 0x0DF8F0,
+        73: 0x0DF8F1,
+        74: 0x0DF8F2,
+        75: 0x0DF8F3,
+        76: 0x0DF8F4,
+        79: 0x0DF8F8,
+        # SM
+        80: 0x0DF908,
+        81: 0x0DF909,
+        84: 0x0DF90A,
+        82: 0x0DF90B,
+        83: 0x0DF90C,
+        85: 0x0DF90D,
+        # FR
+        86: 0x0DF910,
+        87: 0x0DF911,
+        91: 0x0DF918,
+        93: 0x0DF919,
+        92: 0x0DF91A,
+        94: 0x0DF91B,
+        88: 0x0DF920,
+        89: 0x0DF921,
+        90: 0x0DF922,
+        # HS
+        95: 0x0DF928,
+        96: 0x0DF929,
+        99: 0x0DF92A,
+        100: 0x0DF92B,
+        101: 0x0DF930,
+        102: 0x0DF931,
+        103: 0x0DF932,
+        97: 0x0DF938,
+        98: 0x0DF939,
+        # ST
+        104: 0x0DF948,
+        105: 0x0DF949,
+        106: 0x0DF94A,
+        107: 0x0DF94B,
+        109: 0x0DF950,
+        110: 0x0DF951,
+        108: 0x0DF952,
+        114: 0x0DF953,
+        115: 0x0DF954,
+        113: 0x0DF958,
+        111: 0x0DF959,
+        112: 0x0DF95A,
+        # WSW
+        116: 0x0DF960,
+        117: 0x0DF961,
+        118: 0x0DF968,
+        119: 0x0DF969,
+        120: 0x0DF96A,
+        123: 0x0DF970,
+        121: 0x0DF978,
+        122: 0x0DF979,
+        124: 0x0DF980,
+        125: 0x0DF981,
+        # CrC
+        127: 0x0DF988,
+        136: 0x0DF989,
+        126: 0x0DF98A,
+        128: 0x0DF98B,
+        137: 0x0DF98C,
+        129: 0x0DF990,
+        132: 0x0DF993,
+        130: 0x0DF991,
+        131: 0x0DF992,
+        133: 0x0DF998,
+        134: 0x0DF999,
+        135: 0x0DF99A,
+        138: 0x0DF9A8,
+        139: 0x0DF9A9,
+        140: 0x0DF9B0,
+        141: 0x0DF9B1,
+        142: 0x0DF9B2,
+        143: 0x0DF9B8,
+        144: 0x0DF9B9,
+        145: 0x0DF9BA,
+        # CP
+        146: 0x0DF9C8,
+        147: 0x0DF9C9,
+        148: 0x0DF9CA,
+        149: 0x0DF9CB,
+        150: 0x0DF9D0,
+        151: 0x0DF9D1,
+        152: 0x0DF9D2,
+        153: 0x0DF9D8,
+        154: 0x0DF9D9,
+        155: 0x0DF9DA,
+        156: 0x0DF9DB,
+        157: 0x0DF9DC,
+        158: 0x0DF9DD,
+        # SF
+        159: 0x0DF9E0,
+        160: 0x0DF9E1,
+        161: 0x0DF9E8,
+        162: 0x0DF9F0,
+        163: 0x0DF9F8,
+        164: 0x0DF9F9,
+        165: 0x0DF9FA,
+        166: 0x0DFA08,
+        167: 0x0DFA09,
+        168: 0x0DFA10,
+        # TVT
+        169: 0x0DFA18,
+        170: 0x0DFA20,
+        171: 0x0DFA21,
+        172: 0x0DFA28,
+        173: 0x0DFA29,
+        174: 0x0DFA30,
+        175: 0x0DFA31,
+        176: 0x0DFA32,
+        177: 0x0DFA38,
+        178: 0x0DFA39,
+        179: 0x0DFA3A,
+        180: 0x0DFA3B,
+        # MM
+        181: 0x0DFA60,
+        182: 0x0DFA78,
+        183: 0x0DFA80,
+        184: 0x0DFA81,
+        185: 0x0DFA82,
+        186: 0x0DFA88,
+        187: 0x0DFA89,
+        188: 0x0DFA8A,
+        189: 0x0DFA8B,
+        190: 0x0DFA90,
+        191: 0x0DFA98,
+        192: 0x0DFA99,
+        193: 0x0DFAA0,
+        194: 0x0DFAA1,
+        196: 0x0DFAA3,
+        195: 0x0DFAA2,
+        197: 0x0DFAA8,
+        198: 0x0DFAA9,
+        199: 0x0DFAB0,
+        200: 0x0DFAB1,
+        201: 0x0DFAC0,
+        202: 0x0DFAC1,
+        203: 0x0DFAC2,
+        204: 0x0DFAC8,
+    }
+    coinsListLocal = {
+        #Array Order:
+        # CoinNumber : [CoinVisualSprite,CoinHitBoxPosition]
+        # CoinVisualSprite : 4 = Coin Sprite(Not collected),0 = Collected
+        1 : [0x0BF03A,0x0BF03E], # This is the ONLY coin that has an audio bug for w/e reason
+        2 : [0x0BF1D2,0x0BF1D6],
+        3 : [0x0BF15A,0x0BF15E],
+        6 : [0x0BF0CA,0x0BF0CE],
+        7 : [0x0BF1BA,0x0BF1BE],
+        8 : [0x0BF1A2,0x0BF1A6],
+        9 : [0x0BF1EA,0x0BF1EE],
+        11 : [0x0BF172,0x0BF176],
+        12 : [0x0BF09A,0X0BF09E],
+        13 : [0x0BF142,0x0BF146],
+        14 : [0x0BF1A2,0x0BF1A6],
+        17 : [0x0BF142,0x0BF146],
+        21 : [0x0BF15A,0x0BF15E],
+        23 : [0x0BF1A2,0x0BF1A6],
+        24 : [0x0BF052,0x0BF056],
+        25 : [0x0BF142,0x0BF146],
+        28 : [0x0BF0B2,0x0BF0B6],
+        29 : [0x0BF142,0x0BF146],
+        30 : [0x0BF142,0x0BF146],
+        31 : [0x0BF172,0x0BF176],
+        32 : [0x0BF18A,0x0BF18E],
+        34 : [0x0BF112,0x0BF116],
+        35 : [0x0BF232,0x0BF236],
+        37 : [0x0BF0B2,0x0BF0B6],
+        38 : [0x0BF0E2,0x0BF0E6],
+        39 : [0x0BF1EA,0x0BF1EE],
+        40 : [0x0BF15A,0x0BF15E],
+        41 : [0x0BF00A,0x0BF00E],
+        44 : [0x0BF1A2,0x0BF1A6],
+        45 : [0x0BF172,0x0BF176],
+        46 : [0x0BF1BA,0x0BF1BE],
+        49 : [0x0BF142,0x0BF146],
+        50 : [0x0BF1EA,0x0BF1EE],
+        53 : [0x0BF18A,0x0BF18E],
+        54 : [0x0BF1A2,0x0BF1A6],
+        55 : [0x0BF24A,0x0BF24E],
+        58 : [0x0BF082,0x0BF086],
+        59 : [0x0BF18A,0x0BF18E],
+        64 : [0x0BF1EA,0x0BF1EE],
+        66 : [0x0BEFAA,0x0BEFAE],
+        73 : [0x0BF18A,0x0BF18E],
+        74 : [0x0BF09A,0x0BF09E],
+        75 : [0x0BF0E2,0x0BF0E6],
+        77 : [0x0BF12A,0x0BF12E],
+        78 : [0x0BF18A,0x0BF18E],
+        79 : [0x0BF21A,0x0BF21E],
+        80 : [0X0BF142,0X0BF146],
+        82 : [0X0BF1BA,0X0BF1BE],
+        84 : [0x0BF21A,0x0BF21E],
+        85 : [0x0BF0FA,0x0BF0FE],
+    }
+
+
+    CoinHitBoxPositionOff = 0xFFB0
+    localMonkeyHitbox1 = 0x0BBDF4
+    localMonkeyCatchHitBox1 = 0x0BBDF0 # ? Seems to be indicative of Monkeys Hitboxes FOR THE NET
 
     caughtStatus = {
         "Unloaded": 0x00,
@@ -631,7 +1320,7 @@ class RAM:
             32, 33, 34, 37, 42, 35, 36, 38, 41, 43, 39, 40, 44
         },
         6: {
-            49, 51, 45, 47, 50, 46, 52
+            49, 51, 45, 47, 50, 46,48, 52
         },
         7: {
 
@@ -646,10 +1335,10 @@ class RAM:
             69, 70, 71, 77, 78, 72, 73, 74, 75, 76, 79
         },
         11: {
-            80, 81, 84, 85, 82
+            80, 81, 84, 83, 85, 82
         },
         12: {
-            91, 92, 93, 94, 88, 90, 89
+            86, 87, 91, 92, 93, 94, 88, 90, 89
         },
         13: {
             95, 96, 99, 100, 101, 102, 103, 98, 97
@@ -944,6 +1633,7 @@ class RAM:
 
     tempLastReceivedArchipelagoID = 0x0DFBD8  # 4 bytes
     tempKeyCountFromServer = 0x0DFBDC
+
     # Unused 0DFBDD to 0DFBDF
     tempGadgetStateFromServer = 0x0DFBE0  # 2 bytes - 0DFBE1
 
@@ -962,6 +1652,7 @@ class RAM:
     tempTokenCountFromServer = 0x0DFBEC
 
     temp_startingCoinAddress = 0x0DFBF0  # Copy all 64 bytes of coin here while entering Level Select
+    levelselect_coinlock_Address = 0x0DFC60 # To tell the client if we are changing the coin table
     blank_coinTable = 0x00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF
     blank_coinTable2 = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
 
@@ -1035,6 +1726,7 @@ class RAM:
         "MonkeyMashTrap": 0x252,
         "IcyHotPantsTrap": 0x253,
         "StunTrap": 0x254,
+        "CameraRotateTrap": 0x255,
         "RainbowCookie": 0x270,
         "FAKE_OOL_ITEM": 0x999,
 
@@ -1089,6 +1781,33 @@ class RAM:
         "Training": 0x20
     }
 
+    MM_SubLevels_Rooms_Spawns = {
+        #Should spawn you in these rooms,depending on RSR
+        #Take the possible room and warps you to the base Sub-Level Room
+        # "Specter": 0x18,
+        69 : 69,
+        # "S_Jake": 0x19,
+        70 : 70,
+        # "S_Circus": 0x1A,
+        71 : 71,
+        # "S_Coaster": 0x1B,
+        72: 72,
+        73: 72,
+        74: 72,
+        75: 72,
+        76 : 72,
+        # "S_Western Land": 0x1C,
+        77 : 77,
+        # "S_Castle": 0x1D,
+        78 : 79,
+        79 : 79,
+        80 : 79,
+        81 : 79,
+        82 : 79,
+        84 : 79,
+        85 : 79,
+    }
+
     levelAddresses = {
         11: 0xdfc71,
         12: 0xdfc72,
@@ -1212,7 +1931,26 @@ class RAM:
             87
         }
     }
+    MM_roomspersublevel = {
+        24: {
 
+        },
+        25: {
+
+        },
+        26: {
+
+        },
+        27: {
+            72, 75, 76
+        },
+        28: {
+            77
+        },
+        29: {
+            78, 79, 80, 81, 82, 84, 85
+        },
+    }
     roomstostring = {
         1: AEDoor.FF_ENTRY.value,
         2: AEDoor.PO_ENTRY.value,
@@ -1275,7 +2013,7 @@ class RAM:
         61: AEDoor.SF_CONVEYOR_LAVA.value,
         62: AEDoor.SF_MECH_FACTORY.value,
         63: AEDoor.TVT_ENTRY.value,
-        64: AEDoor.TVT_WATER_LOBBY.value,  # This starting room may currently be a softlock.
+        64: AEDoor.TVT_WATER_LOBBY.value,
         65: AEDoor.TVT_LOBBY_OUTSIDE.value,
         66: AEDoor.TVT_TANK_LOBBY.value,
         67: AEDoor.TVT_FAN_TANK.value,
@@ -1295,6 +2033,7 @@ class RAM:
         85: AEDoor.MM_SIDE_ENTRY_OUTSIDE_CASTLE.value,
         87: AEDoor.PPM_ENTRY.value
     }
+    PPMShowCoins = 0x139966 # 0 = no coin icon, 2 = Have coin
 # ========================= Lamps / Doors =========================
 
     lockCamera = 0x162057  # Lock Camera (WsW Gong Room) Locked = 0x80, Free = 0x60
@@ -1548,6 +2287,9 @@ class RAM:
     MM_NatalieDoor_Visual2 = 0x0BFE0F  # Open 0x00
     MM_NatalieDoor_Hitbox = 0x167965  # Open 0x80
 
+    MM_AlertRoom_ButtonPressed = 0x172832 # Not pressed = 0x00, Pressed = 0x01
+    MM_AlertRoom_CutsceneTrigger1 = 0x1728C2 # 0x00 Cutscene pending, 0x02 Cutscene Triggered
+    MM_AlertRoom_BGCanPushButton = 0x1728DA # 0x03 BG cannot push the button, 0x00 BG can push it
 # ===================== Input Related =====================
     BUTTON_BYTE_ADDR_HIGH = 0x0B87A3  # Triggers and Face Buttons (contains bits 8-15 of the 16-bit word)
     BUTTON_BYTE_ADDR_LOW = 0x0B87A2  # D-Pad, Start/Select, L3/R3 (contains bits 0-7 of the 16-bit word)
@@ -1688,11 +2430,8 @@ class RAM:
         0x0F5138: 0x00000000,  # Hoop_AnimationReset
     }
 
-
-    #radarFixAddress = 0x0F5125
-    #hoopFixAddress = 0x0F5124  # 2 bytes
-
     gadgetUseStateAddress = 0x0B20CC
+    CatchingState = 0x0F44A0 # 0x00 Not catching , 0x08 Catching a Monkey
     # 1 = "Net down"
     # 8 = "Net down + can catch"
 
@@ -1719,13 +2458,14 @@ class RAM:
     Warp_State = 0x0F4480 #0x01 Spawning, 0x02 = Warping back to Hub, 0x03 ??, 0x04 Exiting Transition
     Transition_Screen_Progress = 0x0F4482 #0x18 = Full black screen, 0x00 is normal
     TransitionFlag = 0x0E3893 # Different stage of processing the transitions
+    Screen_Fading = 0x0E0930
 
     # To translate Transition ID from doorTransitions Table to which address we need to change for the room
     transitionAddresses = {
         # --Array content--
         # TR_ID: {TargetRoomAddress,TargetDoorAddress}
-        1: [0x154264,0x154268],
-        2: [0x15428C,0x154290],
+        1: [0x154264, 0x154268],
+        2: [0x15428C, 0x154290],
         3: [0x1542B4, 0x1542B8],
         4: [0x1542DC, 0x1542E0],
         5: [0x154304, 0x154308],
@@ -1795,6 +2535,8 @@ class RAM:
     # LevelSelection addresses (Number -1)
     selectedWorldAddress = 0x139BC4
     selectedLevelAddress = 0x139BCC
+    selectedSaveLoadMenuEntry = 0x13AF52
+
     preventRoomOverride = 0x1380DC #False = 0xA6420126, True = 0
     worldIsScrollingRight = 0x139BD9 # 2 bytes: 0xFFFF = you are changing to the next world
     worldScrollToRightDPAD = 0x1381D4 # 2 bytes: Enabled = 0009, Disabled = 0000
@@ -1805,9 +2547,77 @@ class RAM:
     startOfLevelNames = 0x1399E8
     startOfEraNames = 0x139B20
 
-    levelselectFonts = 0x139CF6 # 0x36 = Classic One  0x26 = Current One
+    levelselectFonts = 0x139CF6 # 0x36 = Classic One 0x26 = Current One
     time_attack_Times = 0x0DFD44
 
+    coinsperlevel = {
+        1: {
+            1
+        },
+        2: {
+            2
+        },
+        3: {
+            3
+        },
+        4: {
+            6, 7, 8, 9
+        },
+        5: {
+            11, 12, 13, 14
+        },
+        6: {
+            17
+        },
+        7: { #Stadium Attack
+            95,96,97,98,99
+        },
+        8: {
+            21
+        },
+        9: {
+            23
+        },
+        10: {
+            24,25,28
+        },
+        11: {
+            29
+        },
+        12: {
+            30, 31, 32
+        },
+        13: {
+            34, 35
+        },
+        14: { # Gladiator Attack
+            90,91,92,93,94
+        },
+        15: {
+            37, 38, 39
+        },
+        16: {
+            40, 41, 44
+        },
+        17: {
+            45, 46, 49, 50
+        },
+        20: {
+            53, 54, 55
+        },
+        21: {
+            58, 59
+        },
+        22: {
+            64, 66
+        },
+        24: {
+            73, 74, 75, 77, 78, 79, 80, 82, 84, 85
+        },
+        30: { #PPM
+
+        }
+    }
 # ==================== Kickout Prevention ====================
     localLevelState = 0x0F447E # Same as level state, but can be changed to impact some behaviors (Like Kickout Prevention)
 
@@ -1841,7 +2651,9 @@ class RAM:
     spikeIdleTimer = 0x0EC328 # Put this to 0x0000 to wake up
     spikeGroundStateAddress = 0x0EC23D
     spikeHittableAddress = 0x0EC227
-    spikeUltraInstinctAddress = 0x0EC2E2
+    spikeSuperFlyerUseState = 0x0EC2E0 # 0x00 = SuperFlyer activated
+    # This value is also shared with the Spawn/Return to hub animations, but we don't have a use for that in the rando
+
     spikeSkinPalette = 0x0EC1E5
     spikeColor = 0x0EC2D4
     #spikeColor2 = 0x0EC1E6
@@ -1885,6 +2697,22 @@ class RAM:
         0x0734CC : [4,0xAE630008,0x00000000],  # SpikeZ_PosLock2
         0x0738C8 : [4,0xAE620008,0x00000000],  # SpikeZ_PosLock3
     }
+    Spike_CanMove = 0x0F4450 # 0x00 means can move, 0x01 means cannot move (And every odd bit like 0x03)
+
+    SpecialRoom_CameraMode = 0x0C0798 #0x00 = Fixed, 0x01 = Overhead, 0x02 = StrictFollowCam
+    SpecialRoom_CameraRotateLeft = 0x0C07C4 #0x00 = Not tilted, 0xFF = Tilted
+    SpecialRoom_CameraRotateRight = 0x0C07C5 #0x00 = Not tilted, 0xFF = Tilted
+    Inside_CameraMode = 0x0C0F98 #0x00 = Fixed, 0x01 = Overhead, 0x02 = StrictFollowCam
+    Inside_CameraRotateLeft = 0x0C0FC4 #0x00 = Not tilted, 0xFF = Tilted
+    Inside_CameraRotateRight = 0x0C0FC5 #0x00 = Not tilted, 0xFF = Tilted
+    Boss_CameraMode = 0x0C1798 #0x00 = Fixed, 0x01 = Overhead, 0x02 = StrictFollowCam
+    Boss_CameraRotateLeft = 0x0C17C4 #0x00 = Not tilted, 0xFF = Tilted
+    Boss_CameraRotateRight = 0x0C17C5 #0x00 = Not tilted, 0xFF = Tilted
+    Outside_CameraMode = 0x0C1F98 #0x00 = Fixed, 0x01 = Overhead, 0x02 = StrictFollowCam
+    Outside_CameraRotateLeft = 0x0C1FC4 #0x00 = Not tilted, 0xFF = Tilted
+    Outside_CameraRotateRight = 0x0C1FC5 #0x00 = Not tilted, 0xFF = Tilted
+
+
 
     SPIKE_INVINCIBILITY_ADDR = 0x05E748 # Address for Spike's invincibility flag/state
     SPIKE_GOLDEN_FORM_ADDR = 0x0EC2E2    # Address for Spike's golden visual state flag/model ID
@@ -1947,4 +2775,5 @@ class RAM:
     S2_GlobalCutsceneState = 0x0DFDE4 # 0x05 means you beat Specter previously
     # S1_LArm_Life = 0x14474E
     # S1_RArm_Life = 0x1446B6
-    Specter2CompleteAddress = 0x0E00FD
\ No newline at end of file
+    tempSpecter2CompleteAddress = 0x0DFBEE
+    Specter2CompleteAddress = 0x0E00FE
