@@ -136,6 +136,8 @@ class ApeEscapeWorld(World):
                 self.using_ut = True
                 self.passthrough = self.multiworld.re_gen_passthrough["Ape Escape"]
                 self.options.goal.value = self.passthrough["goal"]
+                self.options.fasttokengoal.value = self.passthrough["fasttokengoal"]
+                self.options.allowcollect.value = self.passthrough["allowcollect"]
                 self.options.requiredtokens.value = self.passthrough["requiredtokens"]
                 self.options.totaltokens.value = self.passthrough["totaltokens"]
                 self.options.tokenlocations.value = self.passthrough["tokenlocations"]
@@ -323,11 +325,12 @@ class ApeEscapeWorld(World):
                 self.options.trapweights[AEItem.GadgetShuffleTrap.value],
                 self.options.trapweights[AEItem.MonkeyMashTrap.value],
                 self.options.trapweights[AEItem.IcyHotPantsTrap.value],
-                self.options.trapweights[AEItem.StunTrap.value]
+                self.options.trapweights[AEItem.StunTrap.value],
+                self.options.trapweights[AEItem.CameraRotateTrap.value]
             ]
             # If custom_trapweights are all zeros, reset to default values
             if not any(y > 0 for y in custom_trapweights):
-                trap_weights = [15, 13, 5, 10, 7]
+                trap_weights = [15, 13, 5, 10, 7, 10]
             else:
                 trap_weights = list(custom_trapweights)
 
@@ -347,6 +350,8 @@ class ApeEscapeWorld(World):
                     self.itempool += [self.create_item_trap(AEItem.MonkeyMashTrap.value)]
                 elif trap_weights[2] < randomTrap <= trap_weights[3]:
                     self.itempool += [self.create_item_trap(AEItem.IcyHotPantsTrap.value)]
+                elif trap_weights[3] < randomTrap <= trap_weights[4]:
+                    self.itempool += [self.create_item_trap(AEItem.CameraRotateTrap.value)]
                 else:
                     self.itempool += [self.create_item_trap(AEItem.StunTrap.value)]
 
@@ -436,9 +441,11 @@ class ApeEscapeWorld(World):
             bytestowrite += self.entranceorder[x].bytes
             bytestowrite.append(0)  # We need a separator byte after each level name.
         #self.firstrooms = orderedfirstroomids
-        #print(f"INIT_FirstRooms{self.firstrooms}")
+
         return {
             "goal": self.options.goal.value,
+            "fasttokengoal": self.options.fasttokengoal.value,
+            "allowcollect": self.options.allowcollect.value,
             "requiredtokens": self.options.requiredtokens.value,
             "totaltokens": self.options.totaltokens.value,
             "tokenlocations": self.options.tokenlocations.value,
@@ -446,7 +453,6 @@ class ApeEscapeWorld(World):
             "infinitejump": self.options.infinitejump.value,
             "superflyer": self.options.superflyer.value,
             "entrance": self.options.entrance.value,
-            #"randomizestartingroom": self.options.randomizestartingroom.option_off,
             "randomizestartingroom": self.options.randomizestartingroom.value,
             "unlocksperkey": self.options.unlocksperkey.value,
             "extrakeys": self.options.extrakeys.value,
@@ -458,10 +464,10 @@ class ApeEscapeWorld(World):
             "shufflewaternet": self.options.shufflewaternet.value,
             "lowoxygensounds": self.options.lowoxygensounds.value,
             "fillerpreset": self.options.fillerpreset.value,
-            "customfillerweights":self.options.customfillerweights.value,
+            "customfillerweights": self.options.customfillerweights.value,
             "trappercentage": self.options.trappercentage.value,
             "trapweights": self.options.trapweights.value,
-            "trapsonreconnect": self.options.trapsonreconnect.value,
+            "trapsonreconnect": list(self.options.trapsonreconnect.value),
             "trap_link": self.options.trap_link.value,
             "itemdisplay": self.options.itemdisplay.value,
             "kickoutprevention": self.options.kickoutprevention.value,
@@ -492,12 +498,21 @@ class ApeEscapeWorld(World):
                 spoiler_handle.write(f"\n  {self.levellist[x].name} ==> {self.entranceorder[x].name}")
             spoiler_handle.write(f"\n")
 
-
-    def generate_output(self, output_directory: str):
-        data = {
-            "slot_data": self.fill_slot_data()
-        }
-        # Remove .apae output because it does nothing, we do everything in RAM
-        # filename = f"{self.multiworld.get_out_file_name_base(self.player)}.apae"
-        # with open(os.path.join(output_directory, filename), 'w') as f:
-        #     json.dump(data, f)
+    #def generate_output(self, output_directory: str):
+        #data = {
+        #    "slot_data": self.fill_slot_data(),
+        #    "location_to_item": {self.location_name_to_id[i.name] : item_table[i.item.name] for i in self.multiworld.get_locations() if not i.is_event},
+        #    "data_package": {
+        #        "data": {
+        #            "games": {
+        #                self.game: {
+        #                    "item_name_to_id": self.item_name_to_id,
+        #                    "location_name_to_id": self.location_name_to_id
+        #                }
+        #            }
+        #        }
+        #    }
+        #}
+        #filename = f"{self.multiworld.get_out_file_name_base(self.player)}.apae"
+        #with open(os.path.join(output_directory, filename), 'w') as f:
+        #    json.dump(data, f)
